@@ -240,6 +240,16 @@ where
             _marker: marker::PhantomData,
         }
     }
+
+    /// Returns a command which, when executed will copy the data contained in [source] into this
+    /// [Buffer].
+    ///
+    /// # Panics
+    ///
+    /// Panics when [source] belongs to a different context.
+    pub fn copy_from_command<S>(&self, source: S) -> CopyCommand<T> where S: CopyFrom<T> {
+        source.command(self.into())
+    }
 }
 
 impl<T> Buffer<MaybeUninit<T>> {
@@ -365,6 +375,18 @@ where
             _marker: marker::PhantomData,
         }
     }
+
+    /// Returns a command which, when executed will copy the elements contained in [source]
+    /// into this [Buffer].
+    ///
+    /// # Panics
+    ///
+    /// Panics when [source] belongs to a different context.
+    ///
+    /// Panics when [source] does not have the same length.
+    pub fn copy_from_command<S>(&self, source: S) -> CopyCommand<[T]> where S: CopyFrom<[T]> {
+        source.command(self.into())
+    }
 }
 
 impl<T> Buffer<[MaybeUninit<T>]> {
@@ -561,6 +583,16 @@ where
             _marker: marker::PhantomData,
         }
     }
+
+    /// Returns a command which, when executed will copy the data contained in [source] into this
+    /// [BufferView].
+    ///
+    /// # Panics
+    ///
+    /// Panics when [source] belongs to a different context.
+    pub fn copy_from_command<S>(&self, source: S) -> CopyCommand<T> where S: CopyFrom<T> {
+        source.command(*self)
+    }
 }
 
 impl<'a, T> BufferView<'a, MaybeUninit<T>> {
@@ -597,6 +629,11 @@ impl<'a, T> BufferView<'a, [T]> {
         self.len
     }
 
+    /// The size in bytes of the viewed buffer region.
+    pub fn size_in_bytes(&self) -> usize {
+        std::mem::size_of::<T>() * self.len()
+    }
+
     /// Returns a [BufferView] on an element or a sub-slice of the elements this [Buffer], depending
     /// on the type of `index`.
     ///
@@ -704,6 +741,18 @@ where
             _marker: marker::PhantomData,
         }
     }
+
+    /// Returns a command which, when executed will copy the elements contained in [source]
+    /// into this [BufferView].
+    ///
+    /// # Panics
+    ///
+    /// Panics when [source] belongs to a different context.
+    ///
+    /// Panics when [source] does not have the same length.
+    pub fn copy_from_command<S>(&self, source: S) -> CopyCommand<[T]> where S: CopyFrom<[T]> {
+        source.command(*self)
+    }
 }
 
 impl<'a, T> BufferView<'a, [MaybeUninit<T>]> {
@@ -1297,6 +1346,88 @@ impl<T> BufferViewMutSliceIndex<T> for RangeTo<usize> {}
 
 impl<T> BufferViewMutSliceIndex<T> for RangeToInclusive<usize> {}
 
+/// Helper trait for [Buffer::copy_from_command] and [BufferView::copy_from_command], implemented
+/// for types that can be copied from.
+pub trait CopyFrom<T> where T: ?Sized {
+    fn command(self, target_view: BufferView<T>) -> CopyCommand<T>;
+}
+
+impl<'a, T> CopyFrom<T> for &'a Buffer<T> {
+    fn command(self, target_view: BufferView<T>) -> CopyCommand<T> {
+        if self.data.context_id != target_view.buffer.data.context_id {
+            panic!("Source and target buffer must belong to the same context.");
+        }
+
+        CopyCommand {
+            target: target_view.buffer.data.clone(),
+            source: self.data.clone(),
+            target_offset_in_bytes: target_view.offset_in_bytes(),
+            source_offset_in_bytes: 0,
+            size_in_bytes: mem::size_of::<T>(),
+            _marker: marker::PhantomData
+        }
+    }
+}
+
+impl<'a, T> CopyFrom<T> for BufferView<'a, T> {
+    fn command(self, target_view: BufferView<T>) -> CopyCommand<T> {
+        if self.buffer.data.context_id != target_view.buffer.data.context_id {
+            panic!("Source and target buffer must belong to the same context.");
+        }
+
+        CopyCommand {
+            target: target_view.buffer.data.clone(),
+            source: self.buffer.data.clone(),
+            target_offset_in_bytes: target_view.offset_in_bytes(),
+            source_offset_in_bytes: self.offset_in_bytes(),
+            size_in_bytes: mem::size_of::<T>(),
+            _marker: marker::PhantomData
+        }
+    }
+}
+
+impl<'a, T> CopyFrom<[T]> for &'a Buffer<[T]> {
+    fn command(self, target_view: BufferView<[T]>) -> CopyCommand<[T]> {
+        if self.data.context_id != target_view.buffer.data.context_id {
+            panic!("Source and target buffer must belong to the same context.");
+        }
+
+        if self.len() != target_view.len() {
+            panic!("Source and target buffer have the same length.");
+        }
+
+        CopyCommand {
+            target: target_view.buffer.data.clone(),
+            source: self.data.clone(),
+            target_offset_in_bytes: target_view.offset_in_bytes(),
+            source_offset_in_bytes: 0,
+            size_in_bytes: mem::size_of::<T>() * self.len(),
+            _marker: marker::PhantomData
+        }
+    }
+}
+
+impl<'a, T> CopyFrom<[T]> for BufferView<'a, [T]> {
+    fn command(self, target_view: BufferView<[T]>) -> CopyCommand<[T]> {
+        if self.buffer.data.context_id != target_view.buffer.data.context_id {
+            panic!("Source and target buffer must belong to the same context.");
+        }
+
+        if self.len() != target_view.len() {
+            panic!("Source and target buffer have the same length.");
+        }
+
+        CopyCommand {
+            target: target_view.buffer.data.clone(),
+            source: self.buffer.data.clone(),
+            target_offset_in_bytes: target_view.offset_in_bytes(),
+            source_offset_in_bytes: self.offset_in_bytes(),
+            size_in_bytes: self.size_in_bytes(),
+            _marker: marker::PhantomData
+        }
+    }
+}
+
 /// Command for uploading data to a [Buffer] or a sub-section of a buffer as viewed by a
 /// [BufferView].
 ///
@@ -1579,6 +1710,62 @@ unsafe impl<T> GpuTask<Connection> for DownloadCommand<[T]> {
     }
 }
 
+/// Command for copying data from one buffer to another.
+///
+/// See [Buffer::copy_from_command] and [BufferView::copy_from_command] for details.
+pub struct CopyCommand<T> where T: ?Sized {
+    target: Arc<BufferData>,
+    source: Arc<BufferData>,
+    target_offset_in_bytes: usize,
+    source_offset_in_bytes: usize,
+    size_in_bytes: usize,
+    _marker: marker::PhantomData<T>
+}
+
+unsafe impl<T> GpuTask<Connection> for CopyCommand<T> where T: ?Sized {
+    type Output = ();
+
+    fn context_id(&self) -> ContextId {
+        ContextId::Id(self.target.context_id)
+    }
+
+    fn progress(&mut self, connection: &mut Connection) -> Progress<Self::Output> {
+        let (gl, state) = unsafe { connection.unpack_mut() };
+
+        unsafe {
+            self.target
+                .id()
+                .unwrap()
+                .with_value_unchecked(|buffer_object| {
+                    state
+                        .bind_copy_write_buffer(Some(&buffer_object))
+                        .apply(gl)
+                        .unwrap();
+                });
+
+            self.source
+                .id()
+                .unwrap()
+                .with_value_unchecked(|buffer_object| {
+                    state
+                        .bind_copy_read_buffer(Some(&buffer_object))
+                        .apply(gl)
+                        .unwrap();
+                });
+        }
+
+        gl.copy_buffer_sub_data_with_i32_and_i32_and_i32(
+            GL::COPY_READ_BUFFER,
+            GL::COPY_WRITE_BUFFER,
+            self.source_offset_in_bytes as i32,
+            self.target_offset_in_bytes as i32,
+            self.size_in_bytes as i32
+        );
+
+        Progress::Finished(())
+    }
+}
+
 trait BufferObjectDropper {
     fn drop_buffer_object(&self, id: JsId);
 }
